@@ -14,8 +14,18 @@ def get_changelog_sections(changelog: dict, changelog_sections: list) -> Iterabl
             yield section
 
 
+def get_hash_link(owner: str, repo_name: str, hash_: str) -> str:
+    url = (
+        f"https://gitlab.com/{owner}/{repo_name}/-/commit/{hash_}"
+        if config.get("hvcs") == "gitlab"
+        else f"https://github.com/{owner}/{repo_name}/commit/{hash_}"
+    )
+    short_hash = hash_[:7]
+    return f"[`{short_hash}`]({url})"
+
+
 def changelog_headers(
-    changelog: dict, changelog_sections: list, **kwargs
+    owner: str, repo_name: str, changelog: dict, changelog_sections: list, **kwargs
 ) -> Optional[str]:
     output = ""
 
@@ -25,16 +35,23 @@ def changelog_headers(
 
         # Add each commit from the section in an unordered list
         for item in changelog[section]:
-            output += "* {0} ({1})\n".format(item[1], item[0])
+            output += f"* {item[1]} ({get_hash_link(owner, repo_name, item[0])})\n"
 
     return output
 
 
-def changelog_table(changelog: dict, changelog_sections: list, **kwargs) -> str:
+def changelog_table(
+    owner: str, repo_name: str, changelog: dict, changelog_sections: list, **kwargs
+) -> str:
     output = "| Type | Change |\n| --- | --- |\n"
 
     for section in get_changelog_sections(changelog, changelog_sections):
-        items = "<br>".join([f"{item[1]} ({item[0]})" for item in changelog[section]])
+        items = "<br>".join(
+            [
+                f"{item[1]} ({get_hash_link(owner, repo_name, item[0])})"
+                for item in changelog[section]
+            ]
+        )
         output += f"| {section.title()} | {items} |\n"
 
     return output
